SystemOrganization addCategory: #'Seaside-Pharo-Tools-Web'!

WAComponent subclass: #WAMemoryInput
	instanceVariableNames: 'value multiplier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAMemoryInput commentStamp: 'pmm 1/23/2009 19:35' prior: 0!
A WAMemoryInput is a dialog that allows the user to enter an amount of memory.

This is a Seaside internal class. Do not rely on it being present. If you need it, copy and paste it.!

!WAMemoryInput class methodsFor: 'instance creation' stamp: 'lr 1/25/2009 15:32'!
bytes: anInteger
	^ self basicNew
		initialize;
		bytes: anInteger;
		yourself! !

!WAMemoryInput class methodsFor: 'instance creation' stamp: 'lr 7/25/2011 19:48'!
new
	^ self bytes: 0! !

!WAMemoryInput methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:06'!
bytes
	^ value * multiplier! !

!WAMemoryInput methodsFor: 'accessing' stamp: 'jf 2/15/2010 23:38'!
bytes: anInteger
	anInteger isZero
		ifTrue: [
			multiplier := 0.
			self value: 0 ]
		ifFalse: [
			multiplier := self unitDictionary keys sorted inject: 1 into: [ :last :each |
					(anInteger \\ each) isZero
						ifTrue: [ each ]
						ifFalse: [ last ] ].
			self value: (anInteger // multiplier) ]! !

!WAMemoryInput methodsFor: 'initialization' stamp: 'pmm 10/29/2008 13:05'!
initialize
	super initialize.
	self value: 0.
	self multiplier: 1! !

!WAMemoryInput methodsFor: 'accessing' stamp: 'pmm 10/29/2008 13:00'!
multiplier
	^ multiplier! !

!WAMemoryInput methodsFor: 'accessing' stamp: 'pmm 10/29/2008 13:00'!
multiplier: anInteger
	multiplier := anInteger! !

!WAMemoryInput methodsFor: 'rendering' stamp: 'jf 2/15/2010 23:38'!
renderContentOn: html 
	html textInput
		value: self value greaseString;
		callback: [ :v | self value: v ].
	html select
		list: self unitDictionary keys sorted;
		labels: [ :each | self unitDictionary at: each ];
		selected: self multiplier;
		callback: [ :m | self multiplier: m ]! !

!WAMemoryInput methodsFor: 'private' stamp: 'lr 7/25/2011 20:06'!
unitDictionary
	^ Dictionary new
		at: 1 put: 'bytes';
		at: 1024 put: 'KiB';
		at: 1024 squared put: 'MiB';
		at: (1024 raisedTo: 3) put: 'GiB';
		yourself! !

!WAMemoryInput methodsFor: 'accessing' stamp: 'pmm 10/29/2008 13:00'!
value
	^ value! !

!WAMemoryInput methodsFor: 'accessing' stamp: 'pmm 10/29/2008 13:00'!
value: anInteger
	value := anInteger! !

WAComponent subclass: #WAScreenshot
	instanceVariableNames: 'button zoom'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAScreenshot class methodsFor: 'testing' stamp: 'avi 2/5/2004 12:57'!
canBeRoot
	^ true! !

!WAScreenshot class methodsFor: 'accessing' stamp: 'lr 8/30/2009 17:40'!
description
	^ 'Pharo Screenshot'! !

!WAScreenshot class methodsFor: 'initialization' stamp: 'jf 10/26/2008 17:34'!
initialize
	WAAdmin register: self asApplicationAt: 'tools/screenshot'! !

!WAScreenshot methodsFor: 'accessing' stamp: 'pmm 9/20/2006 17:48'!
buttonCode
	^ button = #red
		ifTrue: [ 4 ]
		ifFalse: [
			button = #blue
				ifTrue: [ 2 ]
				ifFalse: [ 1 ] ]! !

!WAScreenshot methodsFor: 'actions' stamp: 'lr 8/11/2009 20:10'!
click: aPoint
	| event |
	event := Array new: 8.
	event at: 1 put: 1.
	event at: 2 put: Time millisecondClockValue.
	event at: 3 put: aPoint x.
	event at: 4 put: aPoint y.
	event at: 5 put: self buttonCode.
	event at: 6 put: 0.
	self handleEvent: event.
	event at: 5 put: 0.
	self handleEvent: event.
	button := #red! !

!WAScreenshot methodsFor: 'accessing' stamp: 'pmm 9/20/2006 17:48'!
form
	^ zoom
		ifFalse: [ Display copy ]
		ifTrue: [ self topWindow imageForm ]! !

!WAScreenshot methodsFor: 'actions' stamp: 'lr 8/11/2009 20:10'!
handleEvent: anEvent
	(Sensor respondsTo: #handleEvent:)
		ifTrue: [ Sensor perform: #handleEvent: with: anEvent "Pharo" ]
		ifFalse: [ Sensor perform: #processEvent: with: anEvent "Squeak" ]! !

!WAScreenshot methodsFor: 'initialization' stamp: 'pmm 9/20/2006 17:45'!
initialize
	super initialize.
	button := #red.
	zoom := false! !

!WAScreenshot methodsFor: 'accessing' stamp: 'pmm 9/20/2006 17:49'!
offset
	^ zoom
		ifTrue: [ self topWindow position ]
		ifFalse: [ 0@0 ]! !

!WAScreenshot methodsFor: 'rendering' stamp: 'lr 10/28/2007 14:42'!
renderContentOn: html
	html table: [
		html tableRow: [
			html tableData: [ self renderMouseButtonOn: html ].
			html tableData: [ self renderToggleZoomOn: html ].
			html tableData: [ self renderToggleUIProcessOn: html ] ] ].
	self renderMouseClickOn: html.
	html horizontalRule! !

!WAScreenshot methodsFor: 'rendering' stamp: 'obi 12/31/2009 21:30'!
renderMouseButtonOn: html
	| group |
	html form: [
		html table: [
			html tableRow: [
				html tableData: 'Button: '.
				group := html radioGroup.
				group
					callback: [:value | button := value];
					selected: button.
				#(red yellow blue) do: [ :each |
					html tableData
						style: 'background-color: ', each;
						with: [
							group radioButton value: each ] ].
				html tableData: [
					html submitButton text: 'Set'] ] ] ]! !

!WAScreenshot methodsFor: 'rendering' stamp: 'lr 3/4/2009 12:19'!
renderMouseClickOn: html
	html div: [
		html map
			title: 'Click here';
			callback: [ :point | self click: point + self offset];
			with: [ html image form: self form ] ]! !

!WAScreenshot methodsFor: 'rendering' stamp: 'lr 7/25/2011 18:58'!
renderToggleUIProcessOn: html
	html anchor
		callback: [ self toggleUIProcess ];
		with: self toggleUIText! !

!WAScreenshot methodsFor: 'rendering' stamp: 'pmm 11/1/2007 21:42'!
renderToggleZoomOn: html
	html anchor
		callback: [ self toggleZoom ];
		with: self toggleZoomText! !

!WAScreenshot methodsFor: 'actions' stamp: 'NickAger 4/1/2012 11:13'!
toggleUIProcess
	self uiProcess isSuspended
		ifTrue: [ self uiProcess resume ]
		ifFalse: [ self uiProcess suspend ]! !

!WAScreenshot methodsFor: 'accessing' stamp: 'NickAger 4/1/2012 11:13'!
toggleUIText
	^ self uiProcess isSuspended
		ifFalse: [ 'Suspend UI Process' ]
		ifTrue: [ 'Resume UI Process' ]! !

!WAScreenshot methodsFor: 'actions' stamp: 'pmm 9/20/2006 17:47'!
toggleZoom
	zoom := zoom not! !

!WAScreenshot methodsFor: 'accessing' stamp: 'pmm 11/1/2007 21:42'!
toggleZoomText
	^ zoom
		ifTrue: [ 'Zoom out' ]
		ifFalse: [ 'Zoom in' ]! !

!WAScreenshot methodsFor: 'accessing' stamp: 'avi 2/5/2004 14:18'!
topWindow
	^ SystemWindow classPool at: #TopWindow! !

!WAScreenshot methodsFor: 'accessing' stamp: 'NickAger 4/1/2012 11:13'!
uiProcess
	^ UIManager default uiProcess! !

WAComponent subclass: #WAVMParameterEditor
	instanceVariableNames: 'memoryInput parameterIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAVMParameterEditor commentStamp: 'pmm 11/4/2008 21:43' prior: 0!
A WAVMParameterEditor is an dialog that allows the user to edit a virtual machine parameter.!

!WAVMParameterEditor class methodsFor: 'instance creation' stamp: 'jf 3/13/2009 18:20'!
paramterIndex: anInteger 
	^ self basicNew
		initializeWithParameter: anInteger;
		yourself! !

!WAVMParameterEditor methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:13'!
children
	^ Array with: memoryInput! !

!WAVMParameterEditor methodsFor: 'initialization' stamp: 'pmm 10/31/2008 16:43'!
initialize
	super initialize.
	memoryInput := WAMemoryInput new! !

!WAVMParameterEditor methodsFor: 'initialization' stamp: 'jf 3/13/2009 18:20'!
initializeWithParameter: anInteger
	self initialize.
	parameterIndex := anInteger.
	memoryInput := WAMemoryInput bytes: (SmalltalkImage current vmParameterAt: anInteger)! !

!WAVMParameterEditor methodsFor: 'actions' stamp: 'pmm 11/1/2008 15:59'!
ok
	SmalltalkImage current 
		vmParameterAt: parameterIndex
		put: memoryInput bytes.
	self answer! !

!WAVMParameterEditor methodsFor: 'rendering' stamp: 'pmm 11/1/2008 15:59'!
renderContentOn: html
	html form: [
		html paragraph: [ 
			html render: memoryInput.
			html submitButton
				text: 'OK';
				callback: [ self ok ] ] ]! !

WAComponent subclass: #WAVNCController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAVNCController class methodsFor: 'testing' stamp: 'lr 2/12/2007 21:53'!
canBeRoot
	^ true! !

!WAVNCController class methodsFor: 'accessing' stamp: 'lr 6/10/2007 20:26'!
description
	^ 'VNC Controller'! !

!WAVNCController class methodsFor: 'initialization' stamp: 'jf 10/26/2008 17:34'!
initialize
	(Smalltalk hasClassNamed: #RFBServer)
		ifTrue: [ WAAdmin register: self asApplicationAt: 'tools/vnc' ]! !

!WAVNCController methodsFor: 'accessing' stamp: 'jf 9/30/2009 01:10'!
currentDisplay
	^ self serverInstance displayNumber greaseString! !

!WAVNCController methodsFor: 'testing' stamp: 'lr 7/25/2011 20:13'!
isServerRunning
	^ self serverInstance isRunning! !

!WAVNCController methodsFor: 'accessing' stamp: 'lr 11/21/2006 00:02'!
project
	^ Smalltalk classNamed: #Project! !

!WAVNCController methodsFor: 'rendering' stamp: 'pmm 3/6/2007 21:54'!
renderContentOn: html
	html heading: 'VNC Server'.
	self isServerRunning
		ifTrue: [ self renderStopOn: html ]
		ifFalse: [ self renderStartOn: html ].
	html heading: 'UI Process'.
	self project uiProcess isSuspended
		ifFalse: [ self renderSuspendOn: html ]
		ifTrue: [ self renderResumeOn: html ]! !

!WAVNCController methodsFor: 'rendering' stamp: 'lr 11/21/2006 00:01'!
renderResumeOn: html
	html form: [
		html text: 'UI Process is suspended. '.
		html submitButton
			callback: [ self resume ];
			text: 'Resume' ]! !

!WAVNCController methodsFor: 'rendering' stamp: 'jf 9/30/2009 01:04'!
renderStartOn: html 
	html form: 
		[ html text: 'Start VNC on display: '.
		html textInput
			value: 0;
			callback: [ :value | self startOn: value greaseInteger ].
		html submitButton text: 'Start' ]! !

!WAVNCController methodsFor: 'rendering' stamp: 'lr 11/21/2006 00:01'!
renderStopOn: html
	html form: [
		html text: 'Stop VNC (running on ', self currentDisplay, ') '.
		html submitButton
			callback: [ self stop ];
			text: 'Stop' ]! !

!WAVNCController methodsFor: 'rendering' stamp: 'lr 11/21/2006 00:01'!
renderSuspendOn: html
	html form: [
		html text: 'UI Process is running. '.
		html submitButton
			callback: [ self suspend ];
			text: 'Suspend' ]! !

!WAVNCController methodsFor: 'actions' stamp: 'lr 11/21/2006 00:02'!
resume
	self project uiProcess resume! !

!WAVNCController methodsFor: 'accessing' stamp: 'pmm 3/6/2007 21:51'!
serverClass
	^ Smalltalk classNamed: #RFBServer! !

!WAVNCController methodsFor: 'accessing' stamp: 'pmm 3/6/2007 21:53'!
serverInstance
	^ self serverClass server! !

!WAVNCController methodsFor: 'actions' stamp: 'pmm 3/6/2007 21:56'!
startOn: aNumber
	self resume.
	self serverClass start: aNumber! !

!WAVNCController methodsFor: 'actions' stamp: 'pmm 3/6/2007 21:56'!
stop
	self serverClass stop! !

!WAVNCController methodsFor: 'actions' stamp: 'lr 11/21/2006 00:02'!
suspend
	self project uiProcess suspend.
	self stop! !

WAComponent subclass: #WAVersionUploader
	instanceVariableNames: 'file url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAVersionUploader class methodsFor: 'testing' stamp: 'avi 2/5/2004 12:57'!
canBeRoot
	^ true! !

!WAVersionUploader class methodsFor: 'accessing' stamp: 'lr 6/10/2007 20:27'!
description
	^ 'Monticello Browser'! !

!WAVersionUploader class methodsFor: 'initialization' stamp: 'jf 10/26/2008 17:33'!
initialize
	WAAdmin register: self asApplicationAt: 'tools/versionuploader'! !

!WAVersionUploader methodsFor: 'actions' stamp: 'lr 7/25/2011 20:13'!
loadFile
	| reader version |
	file ifNil: [
		self inform: 'no file selected' onAnswer: [ ].
		^ self ].
	reader := MCVersionReader readerClassForFileNamed: file fileName.
	reader ifNil: [
		self inform: file fileName, ' is not in a known format' onAnswer: [ ].
		^ self ].
	version := reader versionFromStream: file contents asByteArray readStream.
	version load! !

!WAVersionUploader methodsFor: 'actions' stamp: 'lr 7/25/2011 20:13'!
loadURL
	| stream version |
	url asUrl fileName ifEmpty: [
		self inform: 'no valid URL given' onAnswer: [ ].
		^ self ].
	stream := url asUrl retrieveContents content asByteArray readStream .
	version := (MCVersionReader readerClassForFileNamed: url) versionFromStream: stream.
	version load! !

!WAVersionUploader methodsFor: 'actions' stamp: 'lr 5/20/2009 20:15'!
newVersion: aWorkingCopy
	| version stream |
	version := [ aWorkingCopy newVersion ]
		on: MCVersionNameAndMessageRequest
		do: [ :request | request resume: (Array with: request suggestedName with: '') ].
	stream := RWBinaryOrTextStream on: String new.
	version fileOutOn: stream.
	self requestContext respond: [ :response |
		response
			contentType: WAMimeType applicationOctetStream;
			attachmentWithFileName: version fileName;
			nextPutAll: stream contents ]! !

!WAVersionUploader methodsFor: 'rendering' stamp: 'pmm 4/24/2008 07:00'!
renderContentOn: html
	self renderDownloadOn: html.
	self renderLoadFromFileOn: html.
	self renderLoadFromUrlOn: html.
	self renderSaveImageOn: html! !

!WAVersionUploader methodsFor: 'rendering' stamp: 'lr 2/28/2009 22:28'!
renderDownloadOn: html
	html heading level2; with: 'Loaded versions'.
	html unorderedList: [
		self sortedWorkingCopies do: [ :each |
			html listItem: [
				html text: each description.
				html space.
				html anchor
					callback: [ self newVersion: each ];
					with: 'Save' ] ] ]! !

!WAVersionUploader methodsFor: 'rendering' stamp: 'pmm 8/31/2008 15:24'!
renderLoadFromFileOn: html
	html heading level2; with: 'Load version from file'.
	html form
		multipart;
		with: [
		 	html div: [
				html fileUpload
					callback: [ :f | file := f ].
				html space.
				html submitButton
					callback: [ self loadFile ];
					text: 'Load' ] ]! !

!WAVersionUploader methodsFor: 'rendering' stamp: 'pmm 12/17/2008 19:49'!
renderLoadFromUrlOn: html
	html heading level2; with: 'Load version from URL'.
	html form: [
		html div: [
			html textInput
				size: 60;
				value: 'http://'; callback: [ :u | url := u ].
			html space.
			html submitButton
				callback: [ self loadURL ];
				text: 'Load' ] ]! !

!WAVersionUploader methodsFor: 'rendering' stamp: 'lr 7/25/2011 18:59'!
renderSaveImageOn: html
	html anchor
		callback: [ self saveImage ];
		with: 'Save Image'! !

!WAVersionUploader methodsFor: 'actions' stamp: 'pmm 1/26/2007 22:55'!
saveImage
	SmalltalkImage current snapshot: true andQuit: false! !

!WAVersionUploader methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:13'!
sortedWorkingCopies
	^ MCWorkingCopy allManagers asSortedCollection: [ :a :b | a packageName <= b packageName ]! !

!WAVersionUploader methodsFor: 'updating' stamp: 'lr 9/8/2009 12:40'!
updateRoot: anHtmlRoot
	super updateRoot: anHtmlRoot.
	anHtmlRoot title: 'Version Uploader'! !

WAObject subclass: #WAUpTimeTracker
	instanceVariableNames: 'imageBootTime'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAUpTimeTracker commentStamp: 'pmm 3/14/2011 08:06' prior: 0!
WAUpTimeTracker is used to track the TimeStamp when the image last booted.

Access the TimeStamp when the image booted as follows:

	WAUpTimeTracker imageBootTime.

Access the Duration how long ago the image booted as follows:

	WAUpTimeTracker imageUpTime.

At system startup the imageBootTime is (re)set automatically.

This is a work around for Time millisecondClockValue wrapping around.!

!WAUpTimeTracker class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/12/2011 18:07'!
default
	^ Default ifNil: [ Default := self new ]  ! !

!WAUpTimeTracker class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/12/2011 18:08'!
imageBootTime
	^ self default imageBootTime ! !

!WAUpTimeTracker class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/12/2011 18:08'!
imageUpTime
	^ self default imageUpTime ! !

!WAUpTimeTracker class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 3/12/2011 18:08'!
initialize 
	GRPlatform current addToStartUpList: self! !

!WAUpTimeTracker class methodsFor: 'initialization' stamp: 'lr 7/25/2011 19:50'!
startUp: resuming
	resuming ifFalse: [
		"don't reset in case of just saving the image"
		^ self ].
	self default reset! !

!WAUpTimeTracker class methodsFor: 'initialization' stamp: 'pmm 3/14/2011 08:03'!
unload 
	GRPlatform current removeFromShutDownList: self! !

!WAUpTimeTracker methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/12/2011 17:29'!
imageBootTime
	^ imageBootTime ! !

!WAUpTimeTracker methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/12/2011 17:30'!
imageUpTime
	^ TimeStamp now - self imageBootTime! !

!WAUpTimeTracker methodsFor: 'initialization' stamp: 'pmm 3/14/2011 08:02'!
initialize
	super initialize.
	self reset! !

!WAUpTimeTracker methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/12/2011 17:30'!
reset
	imageBootTime := TimeStamp now! !

!GRPackage class methodsFor: '*seaside-pharo-tools-web' stamp: 'lr 2/17/2010 15:13'!
seasidePharoToolsWeb
	^ self new
		name: 'Seaside-Pharo-Tools-Web';
		description: 'Pharo-specific web-based administration tools for Seaside.';
		addDependency: 'Seaside-Tools-Web';
		addDependency: 'Seaside-Pharo-Core';
		addDependency: 'Seaside-Pharo-Canvas';
		url: #seasideUrl;
		yourself! !

WAStatusItem subclass: #WAPharoStatusItem
	instanceVariableNames: 'image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAPharoStatusItem commentStamp: 'pmm 11/4/2008 21:49' prior: 0!
A WAImageStatusItem is the abstract base class for other classes providing information about a specific aspect of this system.!

WAPharoStatusItem subclass: #WAAllocatorStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAAllocatorStatus commentStamp: 'pmm 11/4/2008 21:29' prior: 0!
A WAAllocatorStatus displays information about the memory allocator.!

!WAAllocatorStatus methodsFor: 'actions' stamp: 'pmm 12/26/2010 21:46'!
editIntegerVMParameterAt: anInteger named: aString 
	self 
		request: aString
		default: (SmalltalkImage current vmParameterAt: anInteger)
		onAnswer: [ :value | 
			SmalltalkImage current 
				vmParameterAt: anInteger
				put: value greaseInteger ]! !

!WAAllocatorStatus methodsFor: 'actions' stamp: 'lr 1/24/2009 21:12'!
editMemoryVMParameterAt: anInteger named: aString
	self show: ((WAVMParameterEditor paramterIndex: anInteger)
		addMessage: aString;
		yourself)! !

!WAAllocatorStatus methodsFor: 'accessing' stamp: 'jf 1/19/2009 12:28'!
label
	^ 'Allocator'! !

!WAAllocatorStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 16:08'!
priority
	^ 500! !

!WAAllocatorStatus methodsFor: 'rendering' stamp: 'pmm 11/1/2008 15:53'!
renderEditIntegerParameterAt: anInteger named: aString on: html
	| parameter |
	parameter := SmalltalkImage current vmParameterAt: anInteger.
	self
		renderLabel: aString
		value: [
			html render: parameter.
			self
				renderEditVMParameterAt: anInteger
				callback: [ self editIntegerVMParameterAt: anInteger named: aString ]
				on: html ]
		on: html! !

!WAAllocatorStatus methodsFor: 'rendering' stamp: 'pmm 11/1/2008 15:52'!
renderEditMemoryParameterAt: anInteger named: aString on: html
	| parameter |
	parameter := SmalltalkImage current vmParameterAt: anInteger.
	self
		renderLabel: aString
		value: [
			html render: (self printMemory: parameter).
			self
				renderEditVMParameterAt: anInteger
				callback: [ self editMemoryVMParameterAt: anInteger named: aString ]
				on: html ]
		on: html! !

!WAAllocatorStatus methodsFor: 'rendering' stamp: 'pmm 10/31/2008 20:48'!
renderEditVMParameterAt: anInteger callback: aBlock on: html
	html space.
	html render: $[.
	html anchor
		title: 'edit';
		callback: aBlock;
		with: $e.
	html render: $]! !

!WAAllocatorStatus methodsFor: 'rendering' stamp: 'pmm 11/1/2008 15:57'!
renderValuesOn: html
	| parameters |
	parameters := SmalltalkImage current getVMParameters.
	
	self renderLabel: 'Allocation Count' value: (parameters at: 4) on: html.
	self renderEditIntegerParameterAt: 5 named: 'Allocations Between GCs' on: html.
	self renderEditIntegerParameterAt: 6 named: 'Survivor Count Tenuring Threshold' on: html.
	
	self renderEditMemoryParameterAt: 23 named: 'Extra VM Memory' on: html.
	self renderEditMemoryParameterAt: 24 named: 'Shrinking Threshold' on: html.
	self renderEditMemoryParameterAt: 25 named: 'Memory Headroom' on: html.
	
	self renderLabel: 'Grow Memory Requests' value: (parameters at: 31) on: html.
	self renderLabel: 'Shrink Memory Requests' value: (parameters at: 32) on: html.
	
	self renderLabel: 'Objects Pending Finalization' value: (parameters at: 39) on: html.
	
	self renderLabel: 'Root Table Size' value: (parameters at: 21) on: html.
	self renderLabel: 'Root Table Overflows Since Startup' value: (parameters at: 22) on: html.! !

!WAAllocatorStatus methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:09'!
title
	^ 'Memory Allocator'! !

WAPharoStatusItem subclass: #WAGarbageCollectorStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAGarbageCollectorStatus commentStamp: 'pmm 11/4/2008 21:28' prior: 0!
A WAGarbageCollectorStatus displays information about the garbage collector.
!

!WAGarbageCollectorStatus methodsFor: 'accessing' stamp: 'jf 1/19/2009 12:28'!
label
	^ 'GC'! !

!WAGarbageCollectorStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 16:09'!
priority
	^ 600! !

!WAGarbageCollectorStatus methodsFor: 'rendering' stamp: 'MichaelDavies 8/27/2011 14:42'!
renderCogValuesOn: html
	self renderLabel: 'Cog VM - Size of eden (bytes)' value: (SmalltalkImage current vmParameterAt: 44) on: html.
! !

!WAGarbageCollectorStatus methodsFor: 'rendering' stamp: 'pmm 10/29/2008 12:50'!
renderContentOn: html
	super renderContentOn: html.
	
	html heading level2; with: 'Actions'.
	
	html unorderedList: [
		html listItem: [
			html anchor
				callback: [ Smalltalk garbageCollectMost ];
				title: 'Reclaims recently created garbage (which is usually most of it) fairly quickly.';
				with: 'Young Space GC' ].
		html listItem: [
			html anchor
				callback: [ Smalltalk garbageCollect ];
				title: 'Reclaims all garbage.';
				with: 'Full GC' ].
		html listItem: [
			html anchor
				callback: [ Smalltalk forceTenure ];
				title: 'Tell the GC logic to force a tenure on the next increment GC.';
				with: 'Force Tenure' ].
		html listItem: [
			html anchor
				callback: [ Smalltalk setGCBiasToGrow: 1 ];
				title: 'Indicate that the GC logic should be bias to grow.';
				with: 'Set GC Bias to Grow' ].
		html listItem: [
			html anchor
				callback: [ self setBiasToGrowLimit ];
				title: 'Indicate that the bias to grow logic should do a GC after aNumber Bytes.';
				with: 'Set GC Bias to Grow GC Limit' ] ]! !

!WAGarbageCollectorStatus methodsFor: 'rendering' stamp: 'pmm 10/29/2008 10:41'!
renderHeaderOn: html
	html tableHead: [
		html tableRow: [
			html tableData.
			html tableHeading: 'Count'.
			html tableHeading: 'Total Time'.
			html tableHeading: 'Avg. Time' ] ]! !

!WAGarbageCollectorStatus methodsFor: 'rendering' stamp: 'pmm 12/26/2010 21:49'!
renderValuesOn: html 
	| incrementalGCs tenures parameters fullGCs fullGCTime incrementalGCTime |
	parameters := SmalltalkImage current getVMParameters.
	fullGCs := parameters at: 7.
	fullGCTime := parameters at: 8.
	incrementalGCs := parameters at: 9.
	incrementalGCTime := parameters at: 10.
	tenures := parameters at: 11.
	self 
		renderLabel: 'Full GCs'
		value: fullGCs
		value: (self printMilliSeconds: fullGCTime)
		value: (self printMilliSeconds: fullGCTime dividedBy: fullGCs)
		on: html.
	self 
		renderLabel: 'Incremental GCs'
		value: incrementalGCs
		value: (self printMilliSeconds: incrementalGCTime)
		value: (self printMilliSeconds: incrementalGCTime dividedBy: incrementalGCs)
		on: html.
	self 
		renderLabel: 'Tenures'
		value: tenures
		on: html.
	tenures isZero ifFalse: [
		self 
			renderLabel: 'Avg. GCs/tenure'
			value: (incrementalGCs / tenures) greaseInteger
			on: html ]! !

!WAGarbageCollectorStatus methodsFor: 'actions' stamp: 'jf 9/30/2009 01:04'!
setBiasToGrowLimit
	self 
		request: 'GC Bias To Grow GC Limit (bytes)'
		onAnswer: [ :limit | Smalltalk setGCBiasToGrowGCLimit: limit greaseInteger ]! !

!WAGarbageCollectorStatus methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:09'!
tableBodyCssClass
	^ 'right'! !

!WAGarbageCollectorStatus methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:09'!
title
	^ 'Garbage Collector'! !

WAPharoStatusItem subclass: #WAImageStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAImageStatus commentStamp: 'pmm 11/5/2008 14:26' prior: 0!
A WAImageStatus displays information about the current Smalltalk image.!

!WAImageStatus methodsFor: 'actions' stamp: 'lr 2/16/2010 21:08'!
clear
	WAAdmin clearSessions.
	Smalltalk garbageCollect! !

!WAImageStatus methodsFor: 'accessing' stamp: 'jf 1/19/2009 12:28'!
label
	^ 'Image'! !

!WAImageStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 16:07'!
priority
	^ 100! !

!WAImageStatus methodsFor: 'rendering' stamp: 'MichaelDavies 8/27/2011 11:42'!
renderContentOn: html
	super renderContentOn: html.
	
	html heading level2; with: 'Actions'.
	
	html unorderedList: [
		html listItem: [
			html anchor
				callback: [ self image snapshot: true andQuit: false ];
				with: 'Save' ].
		html listItem: [
			html anchor
				callback: [ self image snapshot: true andQuit: true ];
				with: 'Save and Quit' ].
		html listItem: [
			html anchor
				callback: [ self image snapshot: false andQuit: true ];
				with: 'Quit Without Saving' ] ]! !

!WAImageStatus methodsFor: 'rendering' stamp: 'MichaelDavies 8/27/2011 11:42'!
renderValuesOn: html
	| upTime |
	upTime := Duration milliseconds: Time millisecondClockValue.
	
	self renderLabel: 'Uptime' value: (self printDuration: upTime) on: html.
	self renderLabel: 'Version' value: Smalltalk version on: html.
	self renderLabel: 'Image Path' value: (self image getSystemAttribute: 1) on: html.
	self
		renderLabel: 'Number of Sessions'
		value: [
			html text:self numberOfSessions.
			html space.
			html anchor
				callback: [ self clear ];
				title: 'Expires all sessions and does a full garbage collect.';
				with: 'Clear' ]
		on: html.
	self renderLabel: 'Number of Classes' value: Smalltalk size on: html.! !

!WAImageStatus methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:09'!
title
	^ 'Smalltalk Image'! !

!WAImageStatus methodsFor: 'accessing' stamp: 'pmm 3/14/2011 08:05'!
upTime
	^ WAUpTimeTracker imageUpTime! !

WAPharoStatusItem subclass: #WAJitStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAJitStatus commentStamp: 'pmm 8/28/2011 15:55' prior: 0!
A WAJitStatus displays information about the Just-In-Time native code compiler.!

!WAJitStatus methodsFor: 'testing' stamp: 'pmm 8/28/2011 15:59'!
isSupported
	^ self isRunningCog! !

!WAJitStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 15:55'!
label
	^ 'JIT'! !

!WAJitStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 16:08'!
priority
	^ 400! !

!WAJitStatus methodsFor: 'rendering' stamp: 'pmm 8/28/2011 16:06'!
renderCogValuesOn: html

	self
		renderLabel: 'Size of machine code zone'
		value:  (self printMemory: (self image vmParameterAt: 46)) on: html.
	self
		renderLabel: 'Desired size of machine code zone'
		value:  (self printMemory: (self image vmParameterAt: 47)) on: html.
	
	self
		renderLabel: 'Compiled code compactions since startup'
		value: (self image vmParameterAt: 62) on: html.
	self
		renderLabel: 'Time spent on compiled code compactions since startup'
		value: (self  printMilliSeconds: (self image vmParameterAt: 63)) on: html.! !

!WAJitStatus methodsFor: 'rendering' stamp: 'pmm 8/28/2011 16:04'!
renderValuesOn: html
	"Only Cog values supported"! !

!WAJitStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 16:01'!
title
	^ 'Cog Just-In-Time Compiler'! !

WAPharoStatusItem subclass: #WAMemoryStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAMemoryStatus commentStamp: 'pmm 11/4/2008 21:28' prior: 0!
A WAMemoryStatus displays information about memory usage.!

!WAMemoryStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 16:09'!
priority
	^ 700! !

!WAMemoryStatus methodsFor: 'rendering' stamp: 'pmm 10/29/2008 12:31'!
renderFooterOn: html
	| memoryEnd |
	
	memoryEnd := SmalltalkImage current vmParameterAt: 3.

	html tableFoot
		class: 'right';
		with: [
			self
				renderLabel: 'Total'
				value: (self printMemory: memoryEnd) on: html ]! !

!WAMemoryStatus methodsFor: 'rendering' stamp: 'jf 2/15/2010 23:29'!
renderHeaderOn: html
	html tableHead: [
		html tableRow: [
			html tableData.
			html tableHeading: 'Absolute'.
			html tableHeading: 'Relative' ] ]! !

!WAMemoryStatus methodsFor: 'rendering' stamp: 'pmm 11/3/2008 17:23'!
renderValuesOn: html
	| oldSpaceEnd youngSpaceEnd memoryEnd parameters |
	parameters := SmalltalkImage current getVMParameters.
	
	oldSpaceEnd := parameters first.
	youngSpaceEnd := parameters second.
	memoryEnd := parameters third.
	
	self
		renderLabel: 'Old Space'
		value: (self printMemory: oldSpaceEnd)
		value: (self percent: oldSpaceEnd / memoryEnd)
		on: html.
	self
		renderLabel: 'Young Space'
		value: (self printMemory: (youngSpaceEnd - oldSpaceEnd))
		value: (self percent: youngSpaceEnd - oldSpaceEnd / memoryEnd)
		on: html.
	self
		renderLabel: 'Used'
		value: (self printMemory: youngSpaceEnd)
		value: (self percent: youngSpaceEnd / memoryEnd)
		on: html.
	self
		renderLabel: 'Free'
		value: (self printMemory: (memoryEnd - youngSpaceEnd))
		value: (self percent: memoryEnd - youngSpaceEnd / memoryEnd)
		on: html.! !

!WAMemoryStatus methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:09'!
tableBodyCssClass
	^ 'right'! !

!WAMemoryStatus methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:09'!
title
	^ 'Memory'! !

WAPharoStatusItem subclass: #WAOperatingSystemStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAOperatingSystemStatus commentStamp: 'pmm 11/4/2008 21:29' prior: 0!
A WAOperatingSystemStatus displays information about the operating system the virtual machine of this image runs on.!

!WAOperatingSystemStatus methodsFor: 'accessing' stamp: 'jf 1/19/2009 12:29'!
label
	^ 'OS'! !

!WAOperatingSystemStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 16:10'!
priority
	^ 900! !

!WAOperatingSystemStatus methodsFor: 'rendering' stamp: 'MichaelDavies 8/27/2011 11:43'!
renderValuesOn: html
	self renderLabel: 'Operating System' value: (self image getSystemAttribute: 1001) on: html.
	self renderLabel: 'Version' value: (self image getSystemAttribute: 1002) on: html.
	self renderLabel: 'Processor Type' value: (self image getSystemAttribute: 1003) on: html.! !

!WAOperatingSystemStatus methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:09'!
title
	^ 'Operating System'! !

!WAPharoStatusItem class methodsFor: 'testing' stamp: 'jf 2/5/2010 15:13'!
isAbstract
	^ self name = #WAPharoStatusItem! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'lr 11/14/2008 23:37'!
commandLineArgumentsFrom: aSmalltalkImage 
	| options |
	options := ((3 to: 1000) collect: [ :each | aSmalltalkImage getSystemAttribute: each ]) select: [ :each | each notNil ].
	^ String streamContents: 
		[ :stream | 
		options 
			do: [ :each | stream nextPutAll: each ]
			separatedBy: [ stream space ] ]! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'jf 9/30/2009 00:32'!
days
	^ GRPluggablePrinter on: 
		[ :duration | 
		| days |
		days := duration days.
		days > 0 
			ifTrue: [ (days pluralize: 'day') , ' ' ]
			ifFalse: [ '' ] ]! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'jok 3/10/2009 08:42'!
durationPrinter
	^ self days, self paddedHours, self paddedMinutes, self paddedSeconds! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'pmm 8/28/2011 15:39'!
image
	^ SmalltalkImage current! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'MichaelDavies 8/27/2011 14:51'!
isRunningCog
	^ (self image respondsTo: #isRunningCog) and: [ self image isRunningCog ]! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'lr 7/25/2011 20:09'!
numberOfSessions
	| count |
	count := 0.
	WASession allSubInstancesDo: [ :each | count := count + 1 ].
	^ count! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'pmm 2/13/2011 10:53'!
paddedHours
	^ GRPluggablePrinter on: [ :duration | 
		| hours days |
		hours := duration hours.
		days := duration days.
		(hours isZero and: [ days isZero ]) 
			ifTrue: [ '' ]
			ifFalse: [
				((GRNumberPrinter new padding: $0; digits: 2) print: hours),
					(hours = 1  ifTrue: [ ' hr ' ] ifFalse: [ ' hrs ' ]) ] ]! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'pmm 2/13/2011 10:54'!
paddedMinutes
	^ (GRMappedPrinter 
		block: [ :time | time minutes ]
		next: (GRNumberPrinter new padding: $0; digits: 2)) , ' min '! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'jf 9/30/2009 00:32'!
paddedSeconds
	^ (GRMappedPrinter 
		block: [ :time | time seconds ]
		next: (GRPrinter numberWithAtLeastDigits: 2)) , ' sec'! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'lr 7/25/2011 20:09'!
percent: aDecimal
	^ (aDecimal * 100 roundTo: 0.1) printString , '%'! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'lr 7/25/2011 20:09'!
printDuration: aDuration
	^ self durationPrinter print: aDuration! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'jf 9/30/2009 00:32'!
printMemory: anInteger 
	^ GRPrinter binaryFileSize print: anInteger! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'pmm 6/26/2011 14:32'!
printMilliSeconds: aNumber 
	^ aNumber > 2000 
		ifTrue: [ self printDuration: (Duration milliseconds: aNumber greaseInteger) ]
		ifFalse: [
			aNumber > 2 
				ifTrue: [ aNumber greaseInteger printString , ' ms' ]
				ifFalse: [ (GRNumberPrinter new precision: 2) , ' ms' print: aNumber ] ]! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'pmm 8/15/2010 02:04'!
printMilliSeconds: aNumber dividedBy: divisor
	^ divisor isZero
		ifTrue: [
			"avoid ZeroDivide"
			self printMilliSeconds: 0 ]
		ifFalse: [ self printMilliSeconds: aNumber / divisor ]! !

!WAPharoStatusItem methodsFor: 'rendering' stamp: 'MichaelDavies 8/27/2011 11:36'!
renderCogValuesOn: html
	"override if required"! !

!WAPharoStatusItem methodsFor: 'rendering' stamp: 'MichaelDavies 8/27/2011 14:51'!
renderContentOn: html
	"allow optional Cog parameters to be reported easily"
	html table: [
		self renderHeaderOn: html.
		self renderFooterOn: html.
		html tableBody
			class: self tableBodyCssClass;
			with: [ 
				self renderValuesOn: html.
				self isRunningCog  ifTrue: [
					self renderCogValuesOn: html ] ] ]! !

!WAPharoStatusItem methodsFor: 'private' stamp: 'lr 11/14/2008 23:37'!
vmOptionsFrom: aSmalltalkImage 
	| options |
	options := ((-1 
		to: -1000
		by: -1) collect: [ :each | aSmalltalkImage getSystemAttribute: each ]) select: [ :each | each notNil ].
	^ String streamContents: 
		[ :stream | 
		options 
			do: [ :each | stream nextPutAll: each ]
			separatedBy: [ stream space ] ]! !

WAPharoStatusItem subclass: #WAProcessStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAProcessStatus commentStamp: 'pmm 11/4/2008 21:31' prior: 0!
A WAProcessStatus displays information about the Process instances in this image.!

!WAProcessStatus methodsFor: 'private' stamp: 'lr 7/25/2011 20:09'!
isWaitingOnSemaphore: aProcess
	"answers whether the given Process is waiting on a semaphore"

	^ aProcess suspendingList isKindOf: Semaphore! !

!WAProcessStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 16:10'!
priority
	^ 800! !

!WAProcessStatus methodsFor: 'accessing' stamp: 'jf 2/6/2010 04:37'!
processes
	^ (Process allSubInstances
		reject: [ :each | each isTerminated or: [ each = Processor activeProcess ] ])
		sorted: [ :a :b | a priority >= b priority ] ! !

!WAProcessStatus methodsFor: 'rendering' stamp: 'lr 1/25/2009 15:47'!
renderHeaderOn: html
	html tableHead: [
		html tableRow: [
			html tableHeading: 'Name'.
			html tableHeading: 'Priority'.
			html tableHeading: 'Process'.
			html tableData ] ]! !

!WAProcessStatus methodsFor: 'rendering' stamp: 'jok 3/9/2009 14:17'!
renderInspect: aProcess on: html
	Smalltalk at: #WAInspector ifPresent: [ :inspectorClass |
		inspectorClass current notNil ifTrue: [
			html anchor
				callback: [ self show: (inspectorClass current on: aProcess) ];
				with: 'inspect' ] ]! !

!WAProcessStatus methodsFor: 'rendering' stamp: 'pmm 2/16/2009 21:14'!
renderProcess: aProcess on: html
	html tableRow: [
		self renderLabel: aProcess name on: html.
		self renderValue: aProcess priority on: html.
		self renderValue: aProcess printString on: html.
		self
			renderValue: [ self renderTerminate: aProcess on: html ]
			on: html.
		self
			renderValue: [ self renderSignalSemaphore: aProcess on: html ]
			on: html.
		self
			renderValue: [ self renderInspect: aProcess on: html ]
			on: html ]! !

!WAProcessStatus methodsFor: 'rendering' stamp: 'pmm 2/16/2009 21:13'!
renderSignalSemaphore: aProcess on: html
	(self isWaitingOnSemaphore: aProcess) ifTrue: [
		html anchor
			callback: [ self signalSemaphore: aProcess ];
			with: 'signal semaphore' ]! !

!WAProcessStatus methodsFor: 'rendering' stamp: 'jok 3/9/2009 14:06'!
renderTerminate: aProcess on: html
	html anchor
		callback: [ aProcess terminate ];
		with: 'terminate'! !

!WAProcessStatus methodsFor: 'rendering' stamp: 'pmm 2/16/2009 21:11'!
renderValuesOn: html
	self processes do: [ :each |
		self renderProcess: each on: html ]! !

!WAProcessStatus methodsFor: 'actions' stamp: 'lr 7/25/2011 18:58'!
signalSemaphore: aProcess
	(self isWaitingOnSemaphore: aProcess)
		ifFalse: [ ^ self ].
	"there is a potential race condition here because of check-then-act but we don't care about this"
	[ aProcess suspendingList signal ] fork! !

!WAProcessStatus methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:09'!
title
	^ 'Processes'! !

WAPharoStatusItem subclass: #WASeasideVersionStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WASeasideVersionStatus methodsFor: 'accessing' stamp: 'obi 2/10/2009 21:18'!
label
	^ 'Seaside'! !

!WASeasideVersionStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 16:07'!
priority
	^ 200! !

!WASeasideVersionStatus methodsFor: 'rendering' stamp: 'pmm 8/6/2011 11:50'!
renderPackagesOn: html
	| workingCopies |
	workingCopies :=  [ self sortedWorkingCopies ]
		on: Error
		do: [ :e |
			html render: 'Could not resolve working copies'.
			^ self ].
	html table: [
		html tableHead: [
			html tableRow: [
				html tableHeading: 'Package'.
				html tableHeading: 'Version' ] ].
		workingCopies do: [ :category |
			html tableRow: [
				html tableHeading: category key.
				html tableHeading ].
			category value do: [ :each |
				html tableRow: [
					html tableData: each packageName.
					html tableData: (each ancestry ancestorString copyAfterLast: $-) ] ] ] ]! !

!WASeasideVersionStatus methodsFor: 'rendering' stamp: 'jf 2/15/2010 23:30'!
renderValuesOn: html
	self renderLabel: 'Seaside Version' value: GRPlatform current seasideVersion greaseString on: html.
	self renderLabel: 'Grease Version' value: GRPlatform current versionString on: html.
	self renderLabel: 'Packages' value: [ self renderPackagesOn: html ] on: html! !

!WASeasideVersionStatus methodsFor: 'accessing' stamp: 'jf 9/30/2009 00:19'!
sortedWorkingCopies
	| categories packageNames workingCopies |
	categories := Dictionary new.
	packageNames := GRPackage packages collect: [ :each | each name ].
	workingCopies := MCWorkingCopy allManagers select: [ :each | packageNames includes: each packageName ].
	workingCopies do: 
		[ :each | 
		| category |
		category := categories 
			at: (each packageName copyUpTo: $-)
			ifAbsentPut: 
				[ SortedCollection sortBlock: [ :a :b | a packageName caseInsensitiveLessOrEqual: b packageName ] ].
		category add: each ].
	categories := Array streamContents: 
		[ :stream | 
		categories keysAndValuesDo: [ :key :value | stream nextPut: key -> value ] ].
	^ categories asSortedCollection: [ :a :b | a key caseInsensitiveLessOrEqual: b key ]! !

!WASeasideVersionStatus methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:09'!
title
	^ 'Seaside Packages'! !

WAPharoStatusItem subclass: #WASpaceStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WASpaceStatus methodsFor: 'private' stamp: 'lr 7/25/2011 20:10'!
classesWithManyInstances
	^ Array streamContents: [ :stream |
		Smalltalk globals keysAndValuesDo: [ :key :value|
			((value isKindOf: Class) and: [key == value name]) ifTrue: [
				| class |
				class := Smalltalk globals at: key.
				(self shouldConsider: class) ifTrue: [
					stream nextPut: class ] ] ] ]! !

!WASpaceStatus methodsFor: 'accessing' stamp: 'pmm 10/16/2010 22:38'!
label
	^ 'Space (slow!!)'! !

!WASpaceStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 16:10'!
priority
	^ 1000! !

!WASpaceStatus methodsFor: 'rendering' stamp: 'pmm 10/16/2010 23:08'!
renderValue: anObject on: html
	html tableData
		align: 'right';
		with: anObject! !

!WASpaceStatus methodsFor: 'rendering' stamp: 'pmm 10/16/2010 23:34'!
renderValuesOn: html
	| items totalSpaceForInstances |
	items := self spaceTallyItems.
	totalSpaceForInstances := 0.
	items do: [ :each |
		totalSpaceForInstances := totalSpaceForInstances + each spaceForInstances  ].
	html table: [
		html tableHead: [
			html tableRow: [
				html tableHeading: 'Class'.
				html tableHeading: 'code space'.
				html tableHeading: '# instances'.
				html tableHeading: 'inst space'.
				html tableHeading: 'percent' ] ].
		html tableBody: [
			items do: [ :each |
				html tableRow: [
					html tableHeading align: 'right'; with: each analyzedClassName.
					self renderValue: each codeSize on: html.
					self renderValue: each instanceCount on: html.
					self renderValue: each spaceForInstances on: html.
					self renderValue: ((each spaceForInstances / totalSpaceForInstances) * 100 roundTo: 0.1) on: html  ] ] ] ]! !

!WASpaceStatus methodsFor: 'private' stamp: 'lr 7/25/2011 20:10'!
shouldConsider: aClass
	| count |
	count := 0.
	aClass allInstancesDo: [ :instance |
		count := count + 1.
		count >= 100 ifTrue: [ ^ true ] ].
	^ false! !

!WASpaceStatus methodsFor: 'private' stamp: 'pmm 10/16/2010 23:32'!
spaceTallyItems
	^ (SpaceTally new spaceTally: self classesWithManyInstances)
		sort: [ :a :b | a spaceForInstances > b spaceForInstances ]! !

!WASpaceStatus methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:10'!
title
	^ 'Space Usage per Class'! !

WAPharoStatusItem subclass: #WAVirtualMachineStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Tools-Web'!

!WAVirtualMachineStatus commentStamp: 'pmm 11/4/2008 21:30' prior: 0!
A WAVirtualMachineStatus displays information about the virtual machine that runs this image.!

!WAVirtualMachineStatus methodsFor: 'accessing' stamp: 'jf 1/19/2009 12:29'!
label
	^ 'VM'! !

!WAVirtualMachineStatus methodsFor: 'accessing' stamp: 'pmm 8/28/2011 16:08'!
priority
	^ 300! !

!WAVirtualMachineStatus methodsFor: 'rendering' stamp: 'pmm 8/28/2011 16:03'!
renderCogValuesOn: html
	self renderLabel: 'Cog VM - Interpreter' value: (self image getSystemAttribute: 1007) on: html.
	self renderLabel: 'Cog VM - JIT' value: (self image getSystemAttribute: 1008) on: html.
	self renderLabel: 'Cog VM - Stack pages' value: (self image vmParameterAt: 42) on: html.
	self renderLabel: 'Cog VM - Desired stack pages' value: (self image vmParameterAt: 43) on: html.
	"unclear which way round things go - and all appear to be zero anyway"
	"self renderLabel: 'Cog VM - bit 0' value: ((image vmParameterAt: 48) bitAt: 1) on: html.
	self renderLabel: 'Cog VM - Interpreted method bit used?' value: ((image vmParameterAt: 48) bitAt: 2) on: html.
	self renderLabel: 'Cog VM - preempted methods go to back of queue?' value: ((image vmParameterAt: 48) bitAt: 3) on: html."
	self renderLabel: 'Cog VM - ForceInterruptChecks since startup' value: (self image vmParameterAt: 58) on: html.
	self renderLabel: 'Cog VM - Stack page overflows since startup' value: (self image vmParameterAt: 60) on: html.
	self renderLabel: 'Cog VM - Stack page divorces since startup' value: (self image vmParameterAt: 61) on: html.

! !

!WAVirtualMachineStatus methodsFor: 'rendering' stamp: 'MichaelDavies 8/27/2011 11:38'!
renderValuesOn: html
	self renderLabel: 'VM Version' value: (self image getSystemAttribute: 1004) on: html.
	self renderLabel: 'VM Type' value: ((self image vmParameterAt: 40) * 8) printString, 'bit' on: html.
	self renderLabel: 'Endianness' value: self image endianness on: html.
	
	self renderLabel: 'Image Path' value: (self image getSystemAttribute: 1) on: html.
	
	self renderLabel: 'Document to Open' value: (self image getSystemAttribute: 2) on: html.
	self renderLabel: 'VM Options' value: (self vmOptionsFrom: self image) on: html.
	
	self renderLabel: 'Command Line Arguments' value: (self commandLineArgumentsFrom: self image) on: html.
	self
		renderLabel: 'Builtin Modules'
		value: [
			html unorderedList list: self image listBuiltinModules ]
		on: html.
	self
		renderLabel: 'Loaded Modules'
		value: [
			html unorderedList list: self image listLoadedModules ]
		on: html! !

!WAVirtualMachineStatus methodsFor: 'accessing' stamp: 'lr 7/25/2011 20:10'!
title
	^ 'Virtual Machine'! !
WAScreenshot initialize!
WAVNCController initialize!
WAVersionUploader initialize!
WAUpTimeTracker initialize!
